%bcond_with java

%global pb_cxx 4
%global pb_java 3
%global pb_python 4

Summary:        Protocol Buffers - Google's data interchange format
Name:           protobuf
Version:        24.2
Release:        7%{?dist}
License:        BSD
URL:            https://github.com/protocolbuffers/protobuf
Source0:        %{url}/archive/v%{version}/%{name}-%{version}.tar.gz
Source1:        ftdetect-proto.vim
Source2:        protobuf-init.el

Patch0001:      CVE-2024-2410.patch
#CVE-2024-7254
Patch0002:      CVE-2024-7254-0001-Internal-change.patch
Patch0003:      CVE-2024-7254-0002-Internal-change.patch
Patch0004:      CVE-2024-7254-0003-Add-recursion-check-when-parsing-unknown-fields-in-J.patch
## Fix Build error in abseil-cpp-20250127.0
Patch0005:	0001-Add-dependency-on-absl-layout-and-explicitly-include.patch
Patch0006:	0002-Printer-Include-Abseil-s-str_cat.h-for-absl-AlphaNum.patch

Patch3000:      cmake-shared-utf8_range-build-type.patch
Patch3001:      python-cxx-standard.patch
Patch3002:      protobuf-UnittestRetention.patch

BuildRequires:  gcc-c++ cmake ninja-build pkgconfig
BuildRequires:  emacs zlib-devel gmock-devel gtest-devel abseil-cpp-devel jsoncpp-devel

Provides:       protobuf-cxx = %{pb_cxx}.%{version}
Provides:       protobuf-cxx = %{version}-%{release}
Provides:       protobuf-cpp = %{pb_cxx}.%{version}
Provides:       protobuf-cpp = %{version}-%{release}

%description
Protocol Buffers (a.k.a., protobuf) are Google's language-neutral,
platform-neutral, extensible mechanism for serializing structured
data. 

%package compiler
Summary:        Protocol Buffers compiler
Requires:       %{name} = %{version}-%{release}

%description compiler
This package contains Protocol Buffers compiler for all programming languages.

%package devel
Summary:        Protocol Buffers C++ headers and libraries
Requires:       %{name} = %{version}-%{release}
Requires:       %{name}-compiler = %{version}-%{release}
Requires:       protobuf-lite = %{version}-%{release}
Requires:       zlib-devel pkgconfig
Provides:       protobuf-lite-devel = %{version}-%{release}
Obsoletes:      protobuf-lite-devel < 24.2
Obsoletes:      protobuf-lite-static < 24.2

%description devel
This package contains Protocol Buffers compiler for all languages and
C++ headers and libraries

%package lite
Summary:        Protocol Buffers LITE_RUNTIME libraries

%description lite
Protocol Buffers built with optimize_for = LITE_RUNTIME.
The "optimize_for = LITE_RUNTIME" option causes the compiler to generate
code which only depends libprotobuf-lite, which is much smaller than
libprotobuf but lacks descriptors, reflection, and some other features.

%package vim
Summary:        Vim syntax highlighting for Google Protocol Buffers descriptions
Requires:       vim-filesystem
BuildArch:      noarch

%description vim
This package contains syntax highlighting for Google Protocol Buffers
descriptions in Vim editor

%package -n python3-%{name}
Summary:        Python bindings for Google Protocol Buffers
BuildRequires:  python3-devel
BuildRequires:  python3dist(setuptools)
BuildRequires:  python3dist(wheel)
Conflicts:      %{name}-compiler > %{version}
Conflicts:      %{name}-compiler < %{version}
Provides:       %{name}-python3 = %{version}-%{release}
Provides:       protobuf-python3 = %{pb_python}.%{version}
BuildArch:      noarch

%description -n python3-%{name}
This package contains Python libraries for Google Protocol Buffers

%if %{with java}
%package java
Summary:        Java Protocol Buffers runtime library
BuildRequires:  maven-local
BuildRequires:  mvn(com.google.code.gson:gson)
BuildRequires:  mvn(com.google.guava:guava)
BuildRequires:  mvn(com.google.guava:guava-testlib)
BuildRequires:  mvn(com.google.truth:truth)
BuildRequires:  mvn(junit:junit)
BuildRequires:  mvn(org.apache.felix:maven-bundle-plugin)
BuildRequires:  mvn(org.apache.maven.plugins:maven-antrun-plugin)
BuildRequires:  mvn(org.apache.maven.plugins:maven-source-plugin)
BuildRequires:  mvn(org.codehaus.mojo:build-helper-maven-plugin)
BuildRequires:  mvn(org.easymock:easymock)
BuildRequires:  mvn(org.mockito:mockito-core)
Conflicts:      %{name}-compiler > %{version}
Conflicts:      %{name}-compiler < %{version}
Provides:       protobuf-java = %{version}-%{release}
Provides:       protobuf-java = %{pb_java}.%{version}
BuildArch:      noarch

%description java
This package contains Java Protocol Buffers runtime library.

%package javalite
Summary:        Java Protocol Buffers lite runtime library
BuildArch:      noarch

%description javalite
This package contains Java Protocol Buffers lite runtime library.

%package java-util
Summary:        Utilities for Protocol Buffers
BuildArch:      noarch

%description java-util
Utilities to work with protos. It contains JSON support
as well as utilities to work with proto3 well-known types.

%package javadoc
Summary:        Javadoc for %{name}-java
BuildArch:      noarch

%description javadoc
This package contains the API documentation for %{name}-java.

%package parent
Summary:        Protocol Buffer Parent POM
BuildArch:      noarch

%description parent
Protocol Buffer Parent POM.

%package bom
Summary:        Protocol Buffer BOM POM
BuildArch:      noarch

%description bom
Protocol Buffer BOM POM.

%endif

%package emacs
Summary:        Emacs mode for Google Protocol Buffers descriptions
BuildArch:      noarch
Requires:       emacs-filesystem >= %{_emacs_version}

%description emacs
This package contains syntax highlighting for Google Protocol Buffers
descriptions in the Emacs editor.

%prep
%autosetup -n %{name}-%{version} -p1
find -name \*.cc -o -name \*.h | xargs chmod -x
find examples -type f | xargs chmod 0644
rm -f src/solaris/libstdc++.la

%if %{with java}
%pom_remove_dep com.google.errorprone:error_prone_annotations java/util/pom.xml
%pom_remove_dep com.google.j2objc:j2objc-annotations java/util/pom.xml	
%pom_remove_plugin org.codehaus.mojo:animal-sniffer-maven-plugin java/pom.xml
%pom_remove_plugin org.codehaus.mojo:animal-sniffer-maven-plugin java/util/pom.xml

annotations=$(
    find -name '*.java' |
      xargs grep -h -e '^import com\.google\.errorprone\.annotation' \
                    -e '^import com\.google\.j2objc\.annotations' |
      sort -u | sed 's/.*\.\([^.]*\);/\1/' | paste -sd\|
)
find -name '*.java' | xargs sed -ri \
    "s/^import .*\.($annotations);//;s/@($annotations)"'\>\s*(\((("[^"]*")|([^)]*))\))?//g'

%pom_xpath_inject "pom:configuration/pom:instructions" "<Import-Package>sun.misc;resolution:=optional,*</Import-Package>" java/core
%mvn_file :protobuf-java:jar: %{name}/%{name}-java %{name}

%endif


%build
%cmake \
  -Dprotobuf_BUILD_TESTS=OFF \
  -Dprotobuf_ABSL_PROVIDER:STRING="package" \
  -DABSL_BUILD_TESTING=OFF \
  -DABSL_USE_GOOGLETEST_HEAD=OFF \
  -DABSL_PROPAGATE_CXX_STD=ON \
  -DCMAKE_CXX_STANDARD=17 .

%cmake_build

pushd python
PROTOC="../%{_vpath_builddir}/protoc" \
CXXFLAGS="%{build_cxxflags}" \
LDFLAGS="-L../%{_vpath_builddir} -L../%{_vpath_builddir}/third_party/utf8_range %{build_ldflags}" \
%py3_build %{?with_python_cpp:-- --cpp_implementation}
popd

	
%if %{with java}
%pom_disable_module kotlin java/pom.xml
%pom_disable_module kotlin-lite java/pom.xml
sed -i 's|${protobuf.basedir}/protoc|${protobuf.basedir}/%{_vpath_builddir}/protoc|' java/pom.xml
%mvn_build -s -- -f java/pom.xml
%endif

%{_emacs_bytecompile} editors/protobuf-mode.el

%install
%cmake_install

mkdir -p %{buildroot}%{_emacs_sitelispdir}/%{name}
install -p -m 0644 editors/protobuf-mode.el %{buildroot}%{_emacs_sitelispdir}/%{name}
install -p -m 0644 editors/protobuf-mode.elc %{buildroot}%{_emacs_sitelispdir}/%{name}
mkdir -p %{buildroot}%{_emacs_sitestartdir}
install -p -m 0644 %{SOURCE2} %{buildroot}%{_emacs_sitestartdir}


pushd python
%py3_install
find %{buildroot}%{python3_sitelib} -name \*.py |
  xargs sed -i -e '1{\@^#!@d}'
popd
install -p -m 644 -D %{SOURCE1} %{buildroot}%{_datadir}/vim/vimfiles/ftdetect/proto.vim
install -p -m 644 -D editors/proto.vim %{buildroot}%{_datadir}/vim/vimfiles/syntax/proto.vim

%if %{with java}
%mvn_install
%endif


%check
%ctest


%files
%license LICENSE
%doc CONTRIBUTORS.txt README.md
%{_libdir}/libprotobuf.so.%{version}.0*
%{_libdir}/libutf8_validity.so

%files compiler
%license LICENSE
%doc README.md
%{_bindir}/protoc*
%{_libdir}/libprotoc.so.%{version}.0*

%files devel
%doc examples/README.md examples/add_person.cc examples/addressbook.proto examples/list_people.cc examples/Makefile examples/CMakeLists.txt
%dir %{_includedir}/google
%{_includedir}/google/protobuf/
%{_includedir}/utf8_*.h
%{_libdir}/libprotobuf.so
%{_libdir}/libprotobuf-lite.so
%{_libdir}/libprotoc.so
%{_libdir}/libutf8_range.so
%{_libdir}/cmake/protobuf/
%{_libdir}/cmake/utf8_range/
%{_libdir}/pkgconfig/protobuf.pc
%{_libdir}/pkgconfig/protobuf-lite.pc
%{_libdir}/pkgconfig/utf8_range.pc

%files emacs
%license LICENSE
%{_emacs_sitelispdir}/%{name}/
%{_emacs_sitestartdir}/protobuf-init.el

%files lite
%license LICENSE
%{_libdir}/libprotobuf-lite.so.*

%files vim
%license LICENSE
%{_datadir}/vim/vimfiles/ftdetect/proto.vim
%{_datadir}/vim/vimfiles/syntax/proto.vim

%files -n python3-protobuf
%license LICENSE
%dir %{python3_sitelib}/google
%{python3_sitelib}/google/protobuf/
%{python3_sitelib}/protobuf-%{pb_python}.%{version}-py3.*.egg-info/
%{python3_sitelib}/protobuf-%{pb_python}.%{version}-py3.*-nspkg.pth
%doc python/README.md
%doc examples/add_person.py examples/list_people.py examples/addressbook.proto

%if %{with java}
%files java -f .mfiles-protobuf-java
%license LICENSE
%doc examples/AddPerson.java examples/ListPeople.java
%doc java/README.md

%files java-util -f .mfiles-protobuf-java-util
%license LICENSE

%files javadoc -f .mfiles-javadoc
%license LICENSE

%files parent -f .mfiles-protobuf-parent
%license LICENSE

%files bom -f .mfiles-protobuf-bom
%license LICENSE

%files javalite -f .mfiles-protobuf-javalite
%license LICENSE
%endif

%changelog
* Wed Mar 12 2025 Chunsheng Luo <luffyluo@tencent.com> - 24.2-7
- Rebuild for abseil-cpp-20250127.0

* Fri Oct 11 2024 Chunsheng Luo <luffyluo@tencent.com> - 24.2-6
- Add recursion check when parsing unknown fields in Java(CVE-2024-7254)

* Thu Sep 26 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 24.2-5
- Rebuilt for clarifying the packages requirement in BaseOS and AppStream

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 24.2-4
- Rebuilt for loongarch release

* Wed May 08 2024 Chunsheng Luo <luffyluo@tencent.com> - 24.2-3
- Fix CVE-2024-2410

* Tue Sep 19 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 24.2-2
- Rebuilt for python 3.11

* Mon Sep 18 2023 ZoeDong <zoedong@tencent.com> - 24.2-1
- Bump version to 24.2

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.21.12-2
- Rebuilt for OpenCloudOS Stream 23.09

* Tue Aug 22 2023 jackeyji <jackeyji@tencent.com> - 3.21.12-1
- upgrade to 3.21.12

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.19.6-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.19.6-2
- Rebuilt for OpenCloudOS Stream 23

* Mon Dec 19 2022 ZoeDong <zoedong@tencent.com> - 3.19.6-1
- Initial build
